import unittest
import json
from common.utils import get_message, send_message
from common.variables import ENCODING, ACTION, PRESENCE, TIME, USER, \
    ACCOUNT_NAME, RESPONSE, ERROR
from common.errors import NonDictInputError


class TestConnectSpot:
    """
    Тестовый класс для тестирования отправки и получения
     (при создании требует словарь, который будет
     прогоняться через тестовую функцию)
    """

    def __init__(self, test_dict):
        self.testdict = test_dict

    def send(self, message_to_send):
        """
        Тестовая функция отправки (корректно кодирует сообщение, сораняет то,
         что должно быть отправлено в сокет)
        transmit_message - отправляем в сокет
        :param message_to_send:
        :return:
        """
        json_test_message = json.dumps(self.testdict)
        self.encoded_message = json_test_message.encode(ENCODING)
        self.receved_message = message_to_send

    def recv(self, max_len):
        """
        Получаем данные из сокета
        :param max_len:
        :return:
        """
        json_test_message = json.dumps(self.testdict)
        return json_test_message.encode(ENCODING)


class TestingOfProcess(unittest.TestCase):
    """
    Тестовый класс выполняющий тестирование
    """
    test_dict_send = {
        ACTION: PRESENCE,
        TIME: 111111.111111,
        USER: {
            ACCOUNT_NAME: 'test_test'
        }
    }
    test_dict_gotten_luck = {RESPONSE: 200}
    test_dict_gotten_un_luck = {
        RESPONSE: 400,
        ERROR: 'Bad Request'
    }

    def test_send_message(self):
        """
        Тестируем корректность работы функции отправки,
        создаём тестовый сокет,
        проверяем корректность отправки словаря
        :return:
        """
        # экземпляр тестового словаря, хранит собственно тестовый словарь
        test_socket = TestConnectSpot(self.test_dict_send)
        # вызов тестируемой функции, результаты будут
        # сохранены в тестовом сокете
        send_message(test_socket, self.test_dict_send)
        # проверка корретности кодирования словаря. сравниваем
        # результат довренного кодирования и результат
        # от тестируемой функции
        self.assertEqual(test_socket.encoded_message,
                         test_socket.receved_message)
        # дополнительно, проверим генерацию исключения,
        # при не словаре на входе.
        self.assertRaises(NonDictInputError, send_message, test_socket, 1111)

    def test_listen_message(self):
        """
        Тест функции приёма сообщения
        :return:
        """
        test_sock_luck = TestConnectSpot(self.test_dict_gotten_luck)
        test_sock_un_luck = TestConnectSpot(self.test_dict_gotten_un_luck)
        # тест корректной расшифровки корректного словаря
        self.assertEqual(get_message(test_sock_luck),
                         self.test_dict_gotten_luck)
        # тест корректной расшифровки ошибочного словаря
        self.assertEqual(get_message(test_sock_un_luck),
                         self.test_dict_gotten_un_luck)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'client.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class UiMainClientWindow(object):
    """
    !
    """
    def setup_ui(self, MainClientWindow):
        """
        :param MainClientWindow:
        :return:
        """

        MainClientWindow.setObjectName("MainClientWindow")
        MainClientWindow.resize(788, 756)
        MainClientWindow.setMinimumSize(QtCore.QSize(650, 450))
        MainClientWindow.setAutoFillBackground(False)
        self.centralwidget = QtWidgets.QWidget(MainClientWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.label_contacts = QtWidgets.QLabel(self.centralwidget)
        self.label_contacts.setGeometry(QtCore.QRect(40, 10, 151, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_contacts.setFont(font)
        self.label_contacts.setObjectName("label_contacts")
        self.btn_add_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_add_contact.setGeometry(QtCore.QRect(30, 600, 151, 31))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btn_add_contact.setFont(font)
        self.btn_add_contact.setObjectName("btn_add_contact")
        self.btn_remove_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_remove_contact.setGeometry(QtCore.QRect(30, 640, 141, 31))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btn_remove_contact.setFont(font)
        self.btn_remove_contact.setObjectName("btn_remove_contact")
        self.label_history = QtWidgets.QLabel(self.centralwidget)
        self.label_history.setGeometry(QtCore.QRect(320, 10, 171, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_history.setFont(font)
        self.label_history.setObjectName("label_history")
        self.text_message = QtWidgets.QTextEdit(self.centralwidget)
        self.text_message.setGeometry(QtCore.QRect(310, 500, 441, 71))
        self.text_message.setObjectName("text_message")
        self.label_new_message = QtWidgets.QLabel(self.centralwidget)
        self.label_new_message.setGeometry(QtCore.QRect(320, 440, 400, 41))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_new_message.setFont(font)
        self.label_new_message.setMouseTracking(True)
        self.label_new_message.setObjectName("label_new_message")
        self.list_contacts = QtWidgets.QListView(self.centralwidget)
        self.list_contacts.setGeometry(QtCore.QRect(30, 30, 251, 541))
        self.list_contacts.setObjectName("list_contacts")
        self.list_messages = QtWidgets.QListView(self.centralwidget)
        self.list_messages.setGeometry(QtCore.QRect(310, 30, 441, 381))
        self.list_messages.setObjectName("list_messages")
        self.btn_send = QtWidgets.QPushButton(self.centralwidget)
        self.btn_send.setGeometry(QtCore.QRect(570, 590, 181, 51))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btn_send.setFont(font)
        self.btn_send.setObjectName("btn_send")
        self.btn_clear = QtWidgets.QPushButton(self.centralwidget)
        self.btn_clear.setGeometry(QtCore.QRect(310, 590, 131, 51))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btn_clear.setFont(font)
        self.btn_clear.setObjectName("btn_clear")
        MainClientWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainClientWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 788, 21))
        self.menubar.setObjectName("menubar")
        self.menu = QtWidgets.QMenu(self.menubar)
        self.menu.setObjectName("menu")
        self.menu_2 = QtWidgets.QMenu(self.menubar)
        self.menu_2.setObjectName("menu_2")
        MainClientWindow.setMenuBar(self.menubar)
        self.statusBar = QtWidgets.QStatusBar(MainClientWindow)
        self.statusBar.setObjectName("statusBar")
        MainClientWindow.setStatusBar(self.statusBar)
        self.menu_exit = QtWidgets.QAction(MainClientWindow)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.menu_exit.setFont(font)
        self.menu_exit.setObjectName("menu_exit")
        self.menu_add_contact = QtWidgets.QAction(MainClientWindow)
        self.menu_add_contact.setObjectName("menu_add_contact")
        self.menu_del_contact = QtWidgets.QAction(MainClientWindow)
        self.menu_del_contact.setObjectName("menu_del_contact")
        self.menu.addAction(self.menu_exit)
        self.menu_2.addAction(self.menu_add_contact)
        self.menu_2.addAction(self.menu_del_contact)
        self.menu_2.addSeparator()
        self.menubar.addAction(self.menu.menuAction())
        self.menubar.addAction(self.menu_2.menuAction())

        self.retranslate_ui(MainClientWindow)
        self.btn_clear.clicked.connect(self.text_message.clear)
        QtCore.QMetaObject.connectSlotsByName(MainClientWindow)

    def retranslate_ui(self, MainClientWindow):
        """
        :param MainClientWindow:
        :return:
        """

        _translate = QtCore.QCoreApplication.translate
        MainClientWindow.setWindowTitle(_translate(
            "MainClientWindow", "Чат Программа alpha release"))
        self.label_contacts.setText(_translate("MainClientWindow",
                                               "Список контактов:"))
        self.btn_add_contact.setText(_translate("MainClientWindow",
                                                "Добавить контакт"))
        self.btn_remove_contact.setText(_translate("MainClientWindow",
                                                   "Удалить контакт"))
        self.label_history.setText(_translate("MainClientWindow",
                                              "История сообщений:"))
        self.label_new_message.setText(_translate(
            "MainClientWindow", "<html><head/><body><p>Сообщение "
                                "для:</p></body></html>"))
        self.btn_send.setText(_translate("MainClientWindow",
                                         "Отправить сообщение"))
        self.btn_clear.setText(_translate("MainClientWindow",
                                          "Удалить\nсообщение"))
        self.menu.setTitle(_translate("MainClientWindow", "Файл"))
        self.menu_2.setTitle(_translate("MainClientWindow", "Контакты"))
        self.menu_exit.setText(_translate("MainClientWindow", "Выход"))
        self.menu_add_contact.setText(_translate("MainClientWindow",
                                                 "Добавить контакт"))
        self.menu_del_contact.setText(_translate("MainClientWindow",
                                                 "Удалить контакт"))



from setuptools import setup, find_packages

setup(name="my_server_proj",
      version="0.0.1",
      description="my_server_proj",
      author="Mstislav Agafontsev",
      author_email="slava9mst@mail.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



class IncorrectDataRecivedError(Exception):
    """
    Исключение: от сокета получены некорректные данные.
    """

    def __str__(self):
        return 'Получены некорректные данные от удалённого компьютера.'


class NonDictInputError(Exception):
    """
    Исключение: аргумент функции не словарь.
    """

    def __str__(self):
        return 'Аргумент функции должен быть словарём.'


class ReqFieldMissingError(Exception):
    """
    Ошибка: в принятом словаре отсутствует обязятельное поле.
    """

    def __init__(self, missing_field):
        self.missing_field = missing_field

    def __str__(self):
        return f'В принятом словаре отсутствует обязательное поле ' \
               f'{self.missing_field}.'


class ServerError(Exception):
    """
    Исключение - ошибка сервера
    """

    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text



import sys
import logging
import socket

sys.path.append('../')

# Определение модуля (источника) запуска.
# Метод find() возвращает индекс первого вхождения искомой подстроки,
# если он найден в данной строке.
# Если индекс не найден, модуль возвращает: -1

if sys.argv[0].find('client') == -1:
    logger = logging.getLogger('server')
else:
    logger = logging.getLogger('client')


def log(log_function):
    """
    :param log_function:
    :return:
    """
    def log_ing(*args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        logger.debug(f'Функция: {log_function.__name__} '
                     f'с параметрами: {args} , {kwargs}. '
                     f'Вызов из модуля {log_function.__module__}')
        func_ = log_function(*args, **kwargs)
        return func_
    return log_ing


def login_required(func):
    """
    Декоратор, проверяющий, что клиент авторизован на сервере.
    Проверяет, что передаваемый объект сокета находится в
    списке авторизованных клиентов.
    За исключением передачи словаря-запроса
    на авторизацию. Если клиент не авторизован,
    генерирует исключение TypeError
    """
    def checker(*args, **kwargs):
        """
        Проверяем, что первый аргумент - экземпляр MessageProcessor
        Импортить необходимо тут, иначе ошибка рекурсивного импорта.
        :param args:
        :param kwargs:
        :return:
        """
        from server.core import MessageProcessor
        from client.common.variables import ACTION, PRESENCE
        if isinstance(args[0], MessageProcessor):
            found = False
            for arg in args:
                if isinstance(arg, socket.socket):
                    # Проверяем, что данный сокет есть в списке names класса
                    # MessageProcessor
                    for client in args[0].names:
                        if args[0].names[client] == arg:
                            found = True

            # Теперь надо проверить, что передаваемые аргументы не presence
            # сообщение. Если presense, то разрешаем
            for arg in args:
                if isinstance(arg, dict):
                    if ACTION in arg and arg[ACTION] == PRESENCE:
                        found = True
            # Если не не авторизован и не сообщение начала авторизации, то
            # вызываем исключение.
            if not found:
                raise TypeError
        return func(*args, **kwargs)
    return checker



import ipaddress
import logging
import sys

# Инициализиция логера
# метод определения модуля, источника запуска.
if sys.argv[0].find('client') == -1:
    # если не клиент то сервер!
    logger = logging.getLogger('server')
else:
    # ну, раз не сервер, то клиент
    logger = logging.getLogger('client')


class Port:
    """
    Класс - дескриптор для номера порта.
    """
    def __set__(self, instance, value):
        if value == (range(1024, 65536)):
            logger.critical(
                f'Попытка подключения клиента с неподходящим: '
                f'{value} номером порта.'
                f'Допустимые номера портов с 1024 до 65535. '
                f'Подключение завершается.')
            raise TypeError('Некорректрый номер порта')
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


class Address:
    """
    Класс - дескриптор для IP-адреса.
    """
    def __set__(self, instance, value):
        if value:
            try:
                ip = ipaddress.ip_address(value)
            except ValueError as err_:
                logger.critical(f'Не корректный IP-адрес: {err_}')
                sys.exit(1)
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name



import sys
import json
from common.variables import MAX_PACKAGE_LENGTH, ENCODING
from common.decos import log
sys.path.append('../')


@log
def get_message(client):
    """
    Утилита приёма и декодирования сообщения
     принимает байты выдаёт словарь,
     если приняточто-то другое отдаёт ошибку значения
    :param client:
    :return:
    """
    encoded_response = client.recv(MAX_PACKAGE_LENGTH)
    json_response = encoded_response.decode(ENCODING)
    response = json.loads(json_response)
    if isinstance(response, dict):
        return response
    else:
        raise TypeError


@log
def send_message(sock, message):
    """
    Утилита кодирования и отправки сообщения
    принимает словарь и отправляет его
    :param sock:
    :param message:
    :return:
    """
    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)
    sock.send(encoded_message)



import os
import sys
import logging
import argparse
from Cryptodome.PublicKey import RSA
from PyQt5.QtWidgets import QApplication, QMessageBox
from common.variables import DEFAULT_IP_ADDRESS, DEFAULT_PORT
from common.errors import ServerError
from common.decos import log
from client.database import ClientDatabase
from client.transport import ClientTransport
from client.main_window import ClientMainWindow
from client.start_dialog import UserNameDialog

# Инициализация клиентского логера
logger = logging.getLogger('client')


@log
def create_arg_parser():
    """
    Создание парсера аргументов командной строки
    :return:
    """
    compare = argparse.ArgumentParser()
    compare.add_argument('addr', default=DEFAULT_IP_ADDRESS, nargs='?')
    compare.add_argument('port', default=DEFAULT_PORT, type=int, nargs='?')
    compare.add_argument('-n', '--name', default=None, nargs='?')
    compare.add_argument('-p', '--password', default='', nargs='?')
    area_name = compare.parse_args(sys.argv[1:])
    server_address = area_name.addr
    server_port = area_name.port
    client_name = area_name.name
    client_passwd = area_name.password

    # проверим подходящий номер порта
    if server_port == (range(1024, 65536)):
        logger.critical(
            f'Попытка подключения клиента с неподходящим: {server_port} '
            f'номером порта.'
            f'Допустимые номера портов с 1024 до 65535. '
            f'Подключение завершается.')
        sys.exit(1)

    return server_address, server_port, client_name, client_passwd


# Основная функция клиента
if __name__ == '__main__':
    # Загружаем параметы коммандной строки
    server_address, server_port, client_name, \
        client_passwd = create_arg_parser()
    logger.debug('Аргументы загружены.')

    # Создаём клиентокое приложение
    client_app = QApplication(sys.argv)

    # Если имя пользователя не было указано в командной строке то запросим его
    start_dialog = UserNameDialog()
    if not client_name or not client_passwd:
        client_app.exec_()
        # Если пользователь ввёл имя и нажал ОК, то сохраняем ведённое и
        # удаляем объект, инааче выходим
        if start_dialog.ok_pressed:
            client_name = start_dialog.client_name.text()
            client_passwd = start_dialog.client_passwd.text()
            logger.debug(
                f'Using USERNAME = {client_name}, PASSWD = {client_passwd}.')
        else:
            sys.exit(0)

    logger.info(
        f'Подключен клиент. Адрес сервера пользователя: {server_address}, '
        f'номер порта: {server_port}, имя пользователя: {client_name}')

    # Загружаем ключи из файла, если же файла нет, то генерируем новую пару.
    dir_path = os.getcwd()
    key_file = os.path.join(dir_path, f'{client_name}.key')
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, 'wb') as key:
            key.write(keys.export_key())
    else:
        with open(key_file, 'rb') as key:
            keys = RSA.import_key(key.read())
    logger.debug('Ключи успешно загружены.')

    # Инициализация БД
    database = ClientDatabase(client_name)

    # Создаём объект - транспорт и запускаем транспортный поток
    try:
        transport = ClientTransport(
            server_port,
            server_address,
            database,
            client_name,
            client_passwd,
            keys)
        logger.debug('Объект готов к транспортировке')
    except ServerError as error:
        message = QMessageBox()
        message.critical(start_dialog, 'Ошибка сервера', error.text)
        sys.exit(1)
    transport.setDaemon(True)
    transport.start()

    # Удалим объект диалога за ненадобностью
    del start_dialog

    # Создаём GUI
    main_window = ClientMainWindow(database, transport, keys)
    main_window.make_connection(transport)
    main_window.setWindowTitle(f'Месинджер клиента: {client_name}.')
    client_app.exec_()

    # Раз графическая оболочка закрылась, закрываем транспорт
    transport.transport_shutdown()
    transport.join()
